﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Analysis.SolutionCaching.PermutationEncoding {
  [Item("PermutationCrossoverPointcut", "Wraps a crossover and applies after-crossover actions.")]
  [StorableClass]
  public class PermutationCrossoverPointcut : CrossoverPointcut<IPermutationAdvice, IPermutationCrossover>, IPermutationCrossover, IPermutationPointcut {
    public ILookupParameter<ItemArray<Permutation>> ParentsParameter {
      get { return (ILookupParameter<ItemArray<Permutation>>)Parameters["Parents"]; }
    }
    public ILookupParameter<Permutation> ChildParameter {
      get { return (ILookupParameter<Permutation>)Parameters["Child"]; }
    }

    [StorableConstructor]
    protected PermutationCrossoverPointcut(bool deserializing) : base(deserializing) { }
    protected PermutationCrossoverPointcut(PermutationCrossoverPointcut original, Cloner cloner) : base(original, cloner) { }

    public PermutationCrossoverPointcut()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<Permutation>("Parents", "The parent permutations which should be crossed."));
      ParentsParameter.ActualName = "Permutation";
      Parameters.Add(new LookupParameter<Permutation>("Child", "The child permutation resulting from the crossover."));
      ChildParameter.ActualName = "Permutation";

      foreach (Type t in ApplicationManager.Manager.GetTypes(typeof(IPermutationCrossover))) {
        if ((!typeof(PermutationCrossoverPointcut).IsAssignableFrom(t)) &&
          (!typeof(MultiOperator<IPermutationCrossover>).IsAssignableFrom(t))) {
          CrossoversParameter.ValidValues.Add((IPermutationCrossover)Activator.CreateInstance(t));
        }
      }

      ParameterizeCrossovers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PermutationCrossoverPointcut(this, cloner);
    }

    private void ParameterizeCrossovers() {
      foreach (IPermutationCrossover crossover in CrossoversParameter.ValidValues.OfType<IPermutationCrossover>()) {
        crossover.ChildParameter.ActualName = ChildParameter.Name;
        crossover.ParentsParameter.ActualName = ParentsParameter.Name;
      }
      foreach (IStochasticOperator crossover in CrossoversParameter.ValidValues.OfType<IStochasticOperator>()) {
        crossover.RandomParameter.ActualName = RandomParameter.Name;
      }
    }
  }
}
